
/*
** (c) 2004 Herbert Poetzl
**
** V0.01	ioctls so far
**
*/

#define VERSION "V0.01"

#include <stdio.h>
#include <stdlib.h>
#include <errno.h>
#include <string.h>
#include <sys/ioctl.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>
#include "virtual.h"
 
#ifdef O_LARGEFILE
#define OPEN_FLAGS (O_RDONLY|O_NONBLOCK|O_LARGEFILE)
#else
#define OPEN_FLAGS (O_RDONLY|O_NONBLOCK)
#endif

static	char	err_flag = 0;

static	char	opt_xid = 0;
static	char	opt_flag = 0;
static	char	opt_vers = 0;

static	char *	cmd_name = NULL;

static	int 	num_xid = 0;
static	int 	num_flag = 0;


/* inode ioctls */

#define FIOC_GETXID     _IOR('x', 1, long)
#define FIOC_SETXID     _IOW('x', 2, long)
#define FIOC_SETXIDJ    _IOW('x', 3, long)

#define FIOC_GETXFLG    _IOR('x', 5, long)
#define FIOC_SETXFLG    _IOW('x', 6, long)


int getxid (int fd, int *xid)
{
	int r, c;

	r = ioctl (fd, FIOC_GETXID, &c);
	if (xid)
	    *xid = c;
	return r;
}

int setxid (int fd, int xid)
{
	int r, c;

	c = (int) xid;
	r = ioctl (fd, FIOC_SETXID, &c);
	return r;
}

int getxflg (int fd, int *flag)
{
	int r, c;

	r = ioctl (fd, FIOC_GETXFLG, &c);
	if (flag)
	    *flag = c;
	return r;
}

int setxflg (int fd, int flag)
{
	int r, c;

	c = (int) flag;
	r = ioctl (fd, FIOC_SETXFLG, &c);
	return r;
}


#define OPTIONS "+hf:x:V"

int	main(int argc, char *argv[])
{
        extern int optind;
        extern char *optarg;
        char c, errflg = 0;
	int r;


        cmd_name = argv[0];
        while ((c = getopt(argc, argv, OPTIONS)) != EOF) {
            switch (c) {
            case 'h':
                fprintf(stderr,
                    "This is %s " VERSION "\n"
                    "options are:\n"
                    "-h        print this help message\n"
                    "-f <num>  flag value in decimal\n"
                    "-x <num>  context id\n"
                    "-V        verify interface version\n"
                    "--        end of options\n"
                    ,cmd_name);
                exit(0);
                break;
            case 'f':
                num_flag = atol(optarg);
                opt_flag = 1;
                break;
            case 'x':
                num_xid = atol(optarg);
                opt_xid = 1;
                break;
            case 'V':
                opt_vers = 1;
                break;
            case '?':
            default:
                errflg++;
                break;
            }
        }
        if (errflg) {
            fprintf(stderr, 
                "Usage: %s -[" OPTIONS "]\n"
                "%s -h for help.\n",
                cmd_name, cmd_name);
            exit(2);
        }

	if (opt_vers) {
	    r = vserver(VCMD_get_version, 0, NULL);
	    if (r<0)
	    	perror("vc_get_version");
    	    else 
	    	printf("version: %04x:%04x\n", 
		    (r>>16)&0xFFFF, r&0xFFFF);
	}

        for (; optind < argc; optind++) {
	    struct stat buf;
	    int fd;
	    const char *name;

            name = argv[optind];
    	    if (stat(name, &buf) != 0)
    		perror("stat:");
    	    fd = open(name, OPEN_FLAGS);
	    if (fd<0)
	    	perror("open:");

    	    if (opt_xid)
	    	if (setxid(fd, num_xid))
		    perror("setxid:");
    	    if (opt_flag)
	    	if (setxflg(fd, num_flag))
		    perror("setxflg:");
	    if (getxid(fd, &num_xid))
	    	perror("getxid:");
	    if (getxflg(fd, &num_flag))
	    	perror("getxflg:");
	    printf("xid=%d (%x), flags=%d (%x)\n",
	    	num_xid, num_xid, num_flag, num_flag);
	    close (fd);
	}
	    	
	
	exit((err_flag)?1:0);
}

